/// @file
////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2017/18      Christian Lessig, Otto-von-Guericke Universitaet Magdeburg
///
////////////////////////////////////////////////////////////////////////////////////////////////////
///
///  module     : Tutorial 1
///
///  author     : lessig@isg.cs.ovgu.de
///
///  project    : GPU Programming
///
///  description: caching
///
////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <time.h>


int
main( int argc, char** argv) {

  if( argc < 2) {
    printf( "Error: Specify data size.\n");
    return EXIT_FAILURE;
  }
  unsigned int data_size = (unsigned int) atoi( argv[1]);
  printf( "data_size = %i\n", data_size);

  // allocate data
  int* data = (int*) malloc( sizeof(int) * (data_size*data_size));

  // initialize data
  for( unsigned int i = 0; i < (data_size*data_size); ++i) {
    data[i] = i;
  }

  // start timing
  clock_t tstart = clock();

  // for reliable timing
  for( unsigned int k = 0; k < 1000; ++k) {

    // "process" data
#if 0
    for( unsigned int i = 0; i < data_size; ++i) {
      for( unsigned int j = 0; j < data_size; ++j) {
        data[i*data_size + j] *= data[i*data_size + j];
      }
    }
#else
  // "process" data
  for( unsigned int j = 0; j < data_size; ++j) {
    for( unsigned int i = 0; i < data_size; ++i) {
      data[i*data_size + j] *= data[i*data_size + j];
    }
  }
#endif

  }

  // end timing
  clock_t tend = clock();
  // final time
  double telapsed = (double)(tend - tstart) / CLOCKS_PER_SEC;
  printf( "time elaspsed = %2.4f sec\n", telapsed);

  // cleanup memory
  free( data);

  return EXIT_SUCCESS;
}
